/* Modnumlib Nsp interfacing functions
 * Copyright (C) 2009-2010 Alan Layec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "nsp/interf.h"
#include "modnum_lib.h"

/*
 * y=genint(n(,m,typ,nbit])
 * y=genint(n(,m=m,typ=typ,nbit=nbit])
 *
 * Gnrateur de nombres entiers alatoires (int32)
 *
 * Entres : n,m  : dimensions de la matrice de sortie
 *           typ  : type de codage (dfaut 0)
 *                  0 : codage NRZ
 *                  1 : codage RZ ou nombre non sign si nbit>1
 *                  2 : nombre sign si nbit>1
 *           nbit : longueur en nombre de bits des mots
 *                  de sortie (dfaut 1)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgenint_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *y;
  int m=1;
  int n=1;
  int typ=0;
  int nbit=1;
  int_types T[]={s_int,new_opts,t_end};

  nsp_option opts[]={{"m",s_int,NULLOBJ,-1},
                     {"typ",s_int,NULLOBJ,-1},
                     {"nbit",s_int,NULLOBJ,-1},
                     {NULL,t_end,NULLOBJ,-1}};

  CheckRhs(1,4);
  CheckOptRhs(0,3);
  CheckLhs(1,1);

  if (opt==0) {
    if (GetScalarInt(stack,1,&n) == FAIL) {
      return RET_BUG;
    }
    if (rhs>=2) {
      if (GetScalarInt(stack,2,&m) == FAIL) {
        return RET_BUG;
      }
    }
    if (rhs>=3) {
      if (GetScalarInt(stack,3,&typ) == FAIL) {
        return RET_BUG;
      }
    }
    if (rhs>=4) {
      if (GetScalarInt(stack,4,&nbit) == FAIL) {
        return RET_BUG;
      }
    }
  }
  else {
    if (GetArgs(stack,rhs,opt,T,&n,&opts,&m,&typ,&nbit) == FAIL) {
      return RET_BUG;
    }
  }

  if (n<=0) {
    Scierror("%s: first argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  if (m<=0) {
    Scierror("%s: second argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  if (typ < 0 || typ > 2) {
    Scierror("%s: third argument should be in {0,1,2}\n",NspFname(stack));
    return RET_BUG;
  }
  if (nbit<=0) {
    Scierror("%s: fourth argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',n,m)) == NULLMAT) {
    return RET_BUG;
  }
  if ((y = Mat2int(y)) == NULLMAT) {
    return RET_BUG;
  }

  memset(y->I,0,y->mn*sizeof(int));

  /*Appel genint_c*/
  geninti_c(&n,&m,&nbit,&typ,y->I);

  if ((y = Mat2double(y)) == NULLMAT) {
    return RET_BUG;
  }
  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=genrayl(sig,n[,m])
 * y=genrayl(sig,n[,m=m])
 *
 * Gnrateur de nombres alatoires de Rayleigh
 *
 * Entres : sig  : paramtre (scalaire)
 *           n,m  : dimensions de la matrice de sortie
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgenrayl_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *y;
  double sig=0;
  int m=1;
  int n=1;

  int_types T[]={s_double,s_int,new_opts,t_end};

  nsp_option opts[]={{"m",s_int,NULLOBJ,-1},
                     {NULL,t_end,NULLOBJ,-1}};

  CheckRhs(2,3);
  CheckOptRhs(0,1);
  CheckLhs(1,1);

  if (opt==0) {
    if (GetScalarDouble(stack,1,&sig) == FAIL) {
      return RET_BUG;
    }
    if (GetScalarInt(stack,2,&n) == FAIL) {
      return RET_BUG;
    }
    if (rhs>=3) {
      if (GetScalarInt(stack,3,&m) == FAIL) {
        return RET_BUG;
      }
    }
  }
  else {
    if (GetArgs(stack,rhs,opt,T,&sig,&n,&opts,&m) == FAIL) {
      return RET_BUG;
    }
  }

  if (n<=0) {
    Scierror("%s: second argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  if (m<=0) {
    Scierror("%s: third argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',n,m)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel rnoise_c*/
  rnoise_c(&n,&m,&sig,y->R);

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=upsmpl(u,nech[,typ,init])
 * y=upsmpl(u,nech[,typ=typ,init=init])
 *
 * Sur chantilloneur (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  sur chantillonner
 *                  (int32 ou double) de dimensions n,m
 *           nech : facteur de sur-chantillonnage (scalaire)
 *           typ  : type de sur-chantillonnage (dfaut 0)
 *                  0 : sur-chantillonnage par rptition des valeurs
 *                  1 : sur-chantillonnage par insertion de zro
 *           init : chantillon inital o commence le sur-chantillonnage
 *                  (dfaut 0)
 *
 * Sorties : y : matrice de sortie de taille n*nech,m
 *
 */
int intsurecht_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  int nech=1;
  int typ=0;
  int init=1;
  NspMatrix *y;

  int_types T[]={realmat,s_int,new_opts,t_end};

  nsp_option opts[]={{"typ",s_int,NULLOBJ,-1},
                     {"init",s_int,NULLOBJ,-1},
                     {NULL,t_end,NULLOBJ,-1}};

  CheckRhs(2,4);
  CheckOptRhs(0,2);
  CheckLhs(1,1);

  if (opt==0) {
    if ((u = GetRealMat(stack,1)) == NULLMAT) {
      return RET_BUG;
    }
    if (GetScalarInt(stack,2,&nech) == FAIL) {
      return RET_BUG;
    }
    if (rhs>=3) {
      if (GetScalarInt(stack,3,&typ) == FAIL) {
        return RET_BUG;
      }
    }
    if (rhs>=4) {
      if (GetScalarInt(stack,4,&init) == FAIL) {
        return RET_BUG;
      }
    }
  }
  else {
    if (GetArgs(stack,rhs,opt,T,&u,&nech,&opts,&typ,&init) == FAIL) {
      return RET_BUG;
    }
  }

  if (nech<=0) {
    Scierror("%s: second argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  if (typ != 0 && typ != 1) {
    Scierror("%s: third argument should be 0 or 1\n",NspFname(stack));
    return RET_BUG;
  }
  if (init<0 || init>=nech) {
    Scierror("%s: fourth argument should be positive and less than the second argument.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',u->m*nech,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel surecht_c*/
  surecht_c(&typ,&u->m,&u->n,&nech,&init,u->R,y->R);

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=downsmpl(u,nech[,init])
 * y=downsmpl(u,nech[,init=init])
 *
 * Sous chantilloneur (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  sous chantillonner
 *                  (int32 ou double) de dimensions n,m
 *           nech : facteur de sur-chantillonnage
 *           init : chantillon inital o commence le sur-chantillonnage
 *                  (dfaut 0)
 *
 * Sorties : y : matrice de sortie de taille int(n/nech),m
 *
 */
int intsousecht_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  int nech=1;
  int init=1;
  NspMatrix *y;

  int_types T[]={realmat,s_int,new_opts,t_end};

  nsp_option opts[]={{"init",s_int,NULLOBJ,-1},
                     {NULL,t_end,NULLOBJ,-1}};

  CheckRhs(2,3);
  CheckOptRhs(0,1);
  CheckLhs(1,1);

  if (opt==0) {
    if ((u = GetRealMat(stack,1)) == NULLMAT) {
      return RET_BUG;
    }
    if (GetScalarInt(stack,2,&nech) == FAIL) {
      return RET_BUG;
    }
    if (rhs>=3) {
      if (GetScalarInt(stack,3,&init) == FAIL) {
        return RET_BUG;
      }
    }
  }
  else {
    if (GetArgs(stack,rhs,opt,T,&u,&nech,&opts,&init) == FAIL) {
      return RET_BUG;
    }
  }

  if (nech<=0) {
    Scierror("%s: second argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  if (init<0 || init>=nech) {
    Scierror("%s: third argument should be positive and less than the second argument.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',u->m/nech,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel sousecht_c*/
  sousecht_c(&u->m,&u->n,&nech,&init,u->R,y->R);

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * [yi,yq]=modpsk(u,ns)
 *
 * Modulateur psk (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  moduler par
 *                  tat de phase (int32 ou double) de dimensions n,m
 *           ns   : nombre d'tat de phase
 *
 * Sorties : yi : matrice de sortie de taille n,m composante i (double)
 *           yq : matrice de sortie de taille n,m composante q (double)
 *
 *
 */
int intmodpsk_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  int ns=0;
  NspMatrix *yi;
  NspMatrix *yq;

  CheckRhs(2,2);
  CheckLhs(2,2);

  if ((u = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }
  if (GetScalarInt(stack,2,&ns) == FAIL) {
    return RET_BUG;
  }
  if (ns<=0) {
    Scierror("%s: second argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((yi = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }
  if ((yq = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel modpsk_c*/
  modpsk_c(&u->m,&u->n,&ns,u->R,yi->R,yq->R);

  MoveObj(stack,1, NSP_OBJECT(yi));
  MoveObj(stack,2, NSP_OBJECT(yq));

  return Max(lhs,1);
}

/*
 * [y]=demodpsk(ui,uq,ns)
 *
 * Demodulateur psk (double)
 *
 * Entres : ui   : scalaire, vecteur, matrice  dmoduler par
 *                  tat de phase (double) de dimensions n,m
 *                  composante i
 *           uq   : scalaire, vecteur, matrice  dmoduler par
 *                  tat de phase (double) de dimensions n,m
 *                  composante q
 *           ns   : nombre d'tat de phase
 *
 * Sorties : y : matrice de sortie de taille n,m (double)
 *
 *
 */
int intdemodpsk_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *ui;
  NspMatrix *uq;
  int ns=0;
  NspMatrix *y;

  CheckRhs(3,3);
  CheckLhs(1,1);

  if ((ui = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }
  if ((uq = GetRealMat(stack,2)) == NULLMAT) {
    return RET_BUG;
  }
  CheckSameDims(NspFname(stack), 1, 2, ui, uq);

  if ((GetScalarInt(stack,3,&ns)) == FAIL) {
    return RET_BUG;
  }
  if (ns<=0) {
    Scierror("%s: third argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',ui->m,ui->n)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel demodpsk_c*/
  demodpsk_c(&ui->m,&ui->n,&ns,ui->R,uq->R,y->R);

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * [yi,yq]=modqam(u,ns)
 *
 * Modulateur qam (int32 et double)
 *
 * Entres : u    : scalaire, vecteur, matrice  moduler en
 *                  phase/amplitude (int32 ou double) de dimensions n,m
 *           ns   : nombre d'tat (doit tre un nombre gal  2^nbit avec
                   nbit pair)
 *
 * Sorties : yi : matrice de sortie de taille n,m composante i (int32 ou double)
 *           yq : matrice de sortie de taille n,m composante q (int32 ou double)
 *
 *
 */
int intmodqam_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  int ns;
  NspMatrix *yi;
  NspMatrix *yq;

  int    nb_i;
  double nb_d;

  CheckRhs(2,2);
  CheckLhs(2,2);

  if ((u = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }
  if (GetScalarInt(stack,2,&ns) == FAIL) {
    return RET_BUG;
  }
  if (ns<=0) {
    Scierror("%s: second argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  else {
    nb_d = log(ns)/log(2);
    nb_i = (int) (nb_d/2);
    nb_d = nb_d/2;
    if (nb_d != (double)nb_i) {
      Scierror("%s : number of states must have a value equal "
               "to 2^n with n even.\n",NspFname(stack));
      return RET_BUG;
    }
    nb_i = (int) (nb_d*2);
  }

  if ((yi = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  if ((yq = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel modqam_c*/
  modqam_c(&u->m,&u->n,&nb_i,u->R,yi->R,yq->R);

  MoveObj(stack,1, NSP_OBJECT(yi));
  MoveObj(stack,2, NSP_OBJECT(yq));

  return Max(lhs,1);
}

/*
 * [y]=demodqam(ui,uq,ns)
 *
 * Demodulateur psk (double)
 *
 * Entres : ui   : scalaire, vecteur, matrice  dmoduler par
 *                  phase/amplitude (double ou int32) de dimensions n,m
 *                  composante i
 *           uq   : scalaire, vecteur, matrice  dmoduler par
 *                  phase/amplitude (double ou int32) de dimensions n,m
 *                  composante q
 *           ns   : nombre d'tat (doit tre un nombre gal  2^nbit avec
                   nbit pair)
 *
 * Sorties : y : matrice de sortie de taille n,m (double ou int32)
 *
 */
int intdemodqam_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *ui;
  NspMatrix *uq;
  int ns;
  NspMatrix *y;

  int    nb_i;
  double nb_d;

  CheckRhs(3,3);
  CheckLhs(1,1);

  if ((ui = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }
  if ((uq = GetRealMat(stack,2)) == NULLMAT) {
    return RET_BUG;
  }
  CheckSameDims(NspFname(stack), 1, 2, ui, uq);


  if (GetScalarInt(stack,3,&ns) == FAIL) {
    return RET_BUG;
  }
  if (ns<=0) {
    Scierror("%s: third argument should be positive.\n",NspFname(stack));
    return RET_BUG;
  }
  else {
    nb_d = log(ns)/log(2);
    nb_i = (int) (nb_d/2);
    nb_d = nb_d/2;
    if (nb_d != (double)nb_i) {
      Scierror("%s : number of states must have a value equal "
               "to 2^n with n even.\n",NspFname(stack));
      return RET_BUG;
    }
    nb_i = (int) (nb_d*2);
  }

  if ((y = nsp_matrix_create(NVOID,'r',ui->m,ui->n)) == NULLMAT) {
    return RET_BUG;
  }

  /*Appel demodqam_c*/
  demodqam_c(&ui->m,&ui->n,&nb_i,ui->R,uq->R,y->R);

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=gengold(lreg,ci,coef[,n])
 *
 * gnrateur de squences de gold (int32)
 *
 * Entres : lreg : longueur des registres
 *           ci : conditions initiales
 *                ci = [ci1 ; ci2]
 *           coef : coefficients
 *                  coef = [coef1 ; coef2]
                 les tailles de ci1/ci2 et
 *               de coef1/coef2 doivent tre 1,m
 *           n : tailles des sorties
 *               dfaut : n = 2^(lreg-1)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgengold_c(Stack stack, int rhs, int opt, int lhs)
{
  int lreg=0;
  NspMatrix *ci;
  NspMatrix *coef;
  int n=0;
  NspMatrix *y;

  int ngen; /* nombre de gnrateurs */
  int ly;   /* longueur de la sortie */

  int    *coef1;
  int    *coef2;
  int    *reg1;
  int    *reg2;
  double *ys;

  int i;

  CheckRhs(3,4);
  CheckLhs(1,1);

  if (GetScalarInt(stack,1,&lreg) == FAIL) {
    return RET_BUG;
  }
  else {
    if (lreg <= 0) {
      Scierror("%s: first argument should be positive.\n",NspFname(stack));
      return RET_BUG;
    }
  }

  if ((ci = Mat2int(GetRealMat(stack,2))) == NULLMAT) {
    return RET_BUG;
  }
  if (ci->m !=2) {
    Scierror("%s: second argument must be a matrix.\n",NspFname(stack));
    return RET_BUG;
  }
  ngen = ci->n;

  if ((coef = Mat2int(GetRealMat(stack,3))) == NULLMAT) {
    return RET_BUG;
  }
  if ((coef->m !=2) || (coef->n !=ngen)) {
    Scierror("%s: third argument must be a matrix.\n",NspFname(stack));
    return RET_BUG;
  }

  if (rhs<4) {
    n = (int) pow(2,lreg)-1;
  }
  else {
    if (GetScalarInt(stack,4,&n) == FAIL) {
      return RET_BUG;
    }
    if (n<=0) {
      Scierror("%s: fourth argument should be positive.\n",NspFname(stack));
      return RET_BUG;
    }
  }
  ly = n;

  if ((y = nsp_matrix_create(NVOID,'r',ly,ngen)) == NULLMAT) {
    return RET_BUG;
  }

  /* appel gold_c */
  for(i=0;i<ngen;i++) {
    reg1  = &ci->I[2*i];
    reg2  = &ci->I[2*i+1];
    coef1 = &coef->I[2*i];
    coef2 = &coef->I[2*i+1];
    ys    = &y->R[i*ly];

    gold_c(&lreg, &ly,ys, reg1, reg2, coef1, coef2);
  }

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=genpn(lreg,ci,coef[,n])
 *
 * gnrateur de squences PN (int32)
 *
 * Entres : lreg : longueur des registres
 *           ci : conditions initiales
 *           coef : coefficients
                 les tailles de ci et
 *               de coef doivent tre 1,m
 *           n : tailles des sorties
 *               dfaut : n = 2^(lreg-1)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 */
int intgenpn_c(Stack stack, int rhs, int opt, int lhs)
{
  int lreg=0;
  NspMatrix *ci;
  NspMatrix *coef;
  int n=0;
  NspMatrix *y;

  int ngen; /* nombre de gnrateurs */
  int ly;   /* longueur de la sortie */

  int i;

  CheckRhs(3,4);
  CheckLhs(1,1);

  if (GetScalarInt(stack,1,&lreg) == FAIL) {
    return RET_BUG;
  }
  else {
    if (lreg <= 0) {
      Scierror("%s: first argument should be positive.\n",NspFname(stack));
      return RET_BUG;
    }
  }

  if ((ci = Mat2int(GetRealMat(stack,2))) == NULLMAT) {
    return RET_BUG;
  }
  if (ci->m !=1) {
    Scierror("%s: second argument must be a row vector.\n",NspFname(stack));
    return RET_BUG;
  }
  ngen = ci->n;

  if ((coef = Mat2int(GetRealMat(stack,3))) == NULLMAT) {
    return RET_BUG;
  }
  if ((coef->m !=1) || (coef->n !=ngen)) {
    Scierror("%s: third argument must be a row vector.\n",NspFname(stack));
    return RET_BUG;
  }

  if (rhs<4) {
    n = (int) pow(2,lreg)-1;
  }
  else {
    if (GetScalarInt(stack,4,&n) == FAIL) {
      return RET_BUG;
    }
    if (n<=0) {
      Scierror("%s: fourth argument should be positive.\n",NspFname(stack));
      return RET_BUG;
    }
  }
  ly = n;

  if ((y = nsp_matrix_create(NVOID,'r',ly,ngen)) == NULLMAT) {
    return RET_BUG;
  }

  /* appel mllsrs_c */
  for(i=0;i<ngen;i++) {
    mllsrs_c(&lreg, &ly, &y->R[i*ly], &ci->I[i], &coef->I[i]);
  }

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=spread(u,c)
 *
 * insertion de code
 *
 * Entres : u : vecteur/matrice symbole  taler (de taille n,m)
 *           c : vecteur/matrice code (de taille nc,m)
 *
 * Sorties : y : vecteur/matrice de sortie (de taille n*nc,m)
 *
 */
int intspread_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  NspMatrix *c;
  NspMatrix *y;

  int i;

  CheckRhs(2,2);
  CheckLhs(1,1);

  if ((u = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }

  if ((c = GetRealMat(stack,2)) == NULLMAT) {
    return RET_BUG;
  }

  if (u->n!=c->n) {
    Scierror("%s : symbols input and chips input must have "
             "the same numbers of cols.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',c->m*u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  for (i=0;i<c->n;i++) {
     codinser_c(&c->m,&u->m,   \
                &c->R[i*c->m], \
                &u->R[i*u->m], \
                &y->R[i*c->m*u->m]);
  }

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * y=despread(u,c)
 *
 * deinsertion de code
 *
 * Entres : u : vecteur/matrice symbole  de-taler (de taille n*nc,m)
 *           c : vecteur/matrice code (de taille nc,m)
 *
 * Sorties : y : vecteur/matrice de sortie (de taille n*nc,m)
 *
 */
int intdespread_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  NspMatrix *c;
  NspMatrix *y;

  int i;
  int ns;

  CheckRhs(2,2);
  CheckLhs(1,1);

  if ((u = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }

  if ((c = GetRealMat(stack,2)) == NULLMAT) {
    return RET_BUG;
  }

  ns=u->m/c->m;
  if (ns!=(double)((int)ns)) {
    Scierror("%s : symbols input and chips input must have "
             "doesn't match.Please check.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  for (i=0;i<u->n;i++) {
    decod_c(&c->m,&ns,   \
            &c->R[i*c->m], \
            &u->R[i*u->m], \
            &y->R[i*u->m]);
  }

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

/*
 * [y[,z]]=nfilter(u,h[,z])
 *
 * Banc de filtres  rponse impulsionnelle fini.
 *
 * Entres : u  : vecteur/matrice  filtrer (de taille n,m)
 *           h  : vecteur de la rponse impulsionnelle (de taille nbcoef)
 *
 * Sorties : y : matrice de sortie de taille n,m
 *
 * E/S       z : tats (de taille nbcoef,m) du filtre
 */
int intnfilter_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  NspMatrix *h;
  NspMatrix *z;
  NspMatrix *y;

  CheckRhs(2,3);
  CheckLhs(1,2);

  if ((u = GetRealMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }

  if ((h = GetRealMat(stack,2)) == NULLMAT) {
    return RET_BUG;
  }
  
  if (rhs<3) {
    if ((z = nsp_matrix_create(NVOID,'r',h->m,u->n)) == NULLMAT) {
      return RET_BUG;
    }
    memset(z->R,0.,z->mn*sizeof(double));
  }
  else {
    if ((z = GetRealMat(stack,3)) == NULLMAT) {
      return RET_BUG;
    }
    else {
      if ((h->m!=z->m) || (u->n!=z->n)) {
        Scierror("%s : third argument have bad dimensions.\n",NspFname(stack));
        return RET_BUG;
      } 
    }
  }
  
  if ((y = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }
  
  /*Appel nfilter_c*/
  nfilter_c(&u->m,&u->n,&h->m,u->R,h->R,y->R,z->R);

  MoveObj(stack,1, NSP_OBJECT(y));
  if (lhs==2) {
    MoveObj(stack,2, NSP_OBJECT(z));
  }

  return Max(lhs,1);
}

/*
 * y = awgn(u,sig[,mean])
 * [yi,yq] = awgn(ui,uq,sig[,mean])
 *
 * Canal  bruit blanc gaussien additif
 *
 * Entres : u,[ui,uq] : vecteurs/matrices  bruiter (de taille n,m) (double ou int32)
 *           sig   : variance de la source de bruit blanc (scalaire)
 *           mean  : moyenne de la source de bruit blanc (ou composante DC) (scalaire)
 *
 * Sorties : y,[yi,yq] : matrice(s) de sortie de taille n,m (double)
 *
 */
int intawgn_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  NspMatrix *ui;
  NspMatrix *uq;
  NspMatrix *y;
  NspMatrix *yi;
  NspMatrix *yq;

  double sig=0;
  double mean=0;

  CheckRhs(2,4);
  CheckLhs(1,2);

  if (lhs==2) {
    if (rhs==2) {
      Scierror("%s : With two ouputs, the number of "
               "inputs must at less 3.\n",NspFname(stack));
      return RET_BUG;
    }
    if ((ui = GetRealMat(stack,1)) == NULLMAT) {
      return RET_BUG;
    }
    if ((uq = GetRealMat(stack,2)) == NULLMAT) {
      return RET_BUG;
    }
    CheckSameDims(NspFname(stack), 1, 2, ui, uq);

    if (GetScalarDouble(stack,3,&sig) == FAIL) {
      return RET_BUG;
    }
    if (sig<0) {
      Scierror("%s : third argument should be positive.\n",NspFname(stack));
      return RET_BUG;
    }

    if (rhs==4) {
      if (GetScalarDouble(stack,4,&mean) == FAIL) {
        return RET_BUG;
      }
    }

    if ((yi = nsp_matrix_create(NVOID,'r',ui->m,ui->n)) == NULLMAT) {
      return RET_BUG;
    }
    if ((yq = nsp_matrix_create(NVOID,'r',ui->m,ui->n)) == NULLMAT) {
      return RET_BUG;
    }

    /* appel awgniq_c */
    awgniq_c(&ui->m,&ui->n,&sig,&mean,ui->R,uq->R,yi->R,yq->R);

    MoveObj(stack,1, NSP_OBJECT(yi));
    MoveObj(stack,2, NSP_OBJECT(yq));

    return Max(lhs,1);
  }
  else {
    if (rhs==4) {
      Scierror("%s : With one output, the max number of "
               "inputs must be 3.\n",NspFname(stack));
      return RET_BUG;
    }
    if ((u = GetRealMat(stack,1)) == NULLMAT) {
      return RET_BUG;
    }
    if (GetScalarDouble(stack,2,&sig) == FAIL) {
      return RET_BUG;
    }
    if (sig<0) {
      Scierror("%s : second argument should be positive.\n",NspFname(stack));
      return RET_BUG;
    }

    if (rhs==3) {
      if (GetScalarDouble(stack,3,&mean) == FAIL) {
        return RET_BUG;
      }
    }

    if ((y = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
      return RET_BUG;
    }

    /* appel awgn_c */
    awgn_c(&u->m,&u->n,&sig,&mean,u->R,y->R);

    MoveObj(stack,1, NSP_OBJECT(y));
    return Max(lhs,1);
  }
}

/*
 * [y[,z[,initc]]] = intsym(u,nech[,initc,step,z])
 * [y[,z[,initc]]] = intsym(u,nech[,initc=initc,step=step,z=z])
 *
 * Intgrateur discret avec remise  zro
 *
 * Entres : u     : vecteurs/matrices  intgrer (de taille n,m) (double ou int32)
 *           nech  : nombre d'chantillons par symbole (scalaire)
 *           initc : chantillons initiaux o l'intgration commence
 *                   compris entre 1 et nech (vecteur de taille m)
 *                   (defaut 1)
 *           step  : pas d'intgration (scalaire) (defaut 1)
 *
 * Sorties : y : matrice(s) de sortie de taille n,m (double)
 *
 * Entre/Sortie : z : tat du filtre (matrice de taille m,2 - double)
 *
 */
int intintsym_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  int nech;
  NspMatrix *initc;
  double step=1.;
  NspMatrix *y;
  NspMatrix *z;
  int_types T[]={realmat,s_int,new_opts,t_end};

  nsp_option opts[]={{"initc",realmat,NULLOBJ,-1},
                     {"step",realmat,NULLOBJ,-1},
                     {"z",realmat,NULLOBJ,-1},
                     {NULL,t_end,NULLOBJ,-1}};

  CheckRhs(2,5);
  CheckOptRhs(0,3);
  CheckLhs(1,3);

  if (opt==0) {
    if ((u = GetRealMat(stack,1)) == NULLMAT) {
      return RET_BUG;
    }

    if (GetScalarInt(stack,2,&nech) == FAIL) {
      return RET_BUG;
    }

    if(rhs>=3) {
      if ((initc = Mat2int(GetRealMat(stack,3))) == NULLMAT) {
        return RET_BUG;
      }
    }
    else {
      if ((initc = Mat2int(nsp_matrix_create(NVOID,'r',u->n,1))) == NULLMAT) {
        return RET_BUG;
      }
      memset(initc->I,0,initc->mn*sizeof(int));
    }

    if(rhs>=4) {
      if (GetScalarDouble(stack,4,&step) == FAIL) {
        return RET_BUG;
      }
    }

    if(rhs>=5) {
      if ((z = GetRealMat(stack,5)) == NULLMAT) {
        return RET_BUG;
      }
    }
    else {
      if ((z = nsp_matrix_create(NVOID,'r',u->n,2)) == NULLMAT) {
        return RET_BUG;
      }
      memset(z->R,0.,z->mn*sizeof(double));
    }
  }
  else {
    if (GetArgs(stack,rhs,opt,T,&u,&nech,&opts,&initc,&step,&z) == FAIL) {
      return RET_BUG;
    }
  }

  if (initc->m*initc->n!=u->n) {
    Scierror("%s : third argument have bad dimensions.\n",NspFname(stack));
    return RET_BUG;
  }

  if (z->m*z->n!=u->n*2) {
    Scierror("%s : fifth argument have bad dimensions.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((y = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  /* appel intsym_c */
  intsym_c(&u->m,&u->n,&nech,initc->I,&step,u->R,y->R,z->R);

  /* TODO : z and init_c at the output */
  MoveObj(stack,1, NSP_OBJECT(y));
  return Max(lhs,1);
}


/*
 * [y] = calc_fft(u[,isn])
 *
 * Calcul de transforme de fourier
 *
 * Entres : u : vecteurs/matrices  transformer (de taille n,m) (double ou complexe)
 *           isn : signe de la transforme (-1 directe, 1 inverse)
 *
 * Sorties : y : matrice de sortie de taille n,m (double ou complexe)
 *
 */
int intcalc_fft_c(Stack stack, int rhs, int opt, int lhs)
{
  NspMatrix *u;
  NspMatrix *y;
  NspMatrix *u_r;
  NspMatrix *u_i;
  int isn=-1;
  int i;
  fft_pr_struct fft_pr;

  CheckRhs(1,2);
  CheckLhs(1,1);

  if ((u = GetMat(stack,1)) == NULLMAT) {
    return RET_BUG;
  }

  if (rhs==2) {
    if (GetScalarInt(stack,2,&isn) == FAIL) {
      return RET_BUG;
    }
  }
  if ((isn!=-1)&&(isn!=1)) {
    Scierror("%s : second argument must be -1 or 1.\n",NspFname(stack));
    return RET_BUG;
  }

  if ((u_r = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }
  if ((u_i = nsp_matrix_create(NVOID,'r',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  if (u->rc_type=='r') {
    memcpy(u_r->R,u->R,u->mn*sizeof(double));
    memset(u_i->R,0.,u->mn*sizeof(double));
  }
  else {
    for(i=0;i<u->mn;i++) {
      u_r->R[i]=u->C[i].r;
      u_i->R[i]=u->C[i].i;
    }
  }

  /* initialize fft_pr*/
  fft_pr.lfft   = u->mn;
  fft_pr.isn    = isn;
  fft_pr.ffttyp = 2;

  /* appel calc_fft_c */
  calc_fft_c(&fft_pr,u_r->R,u_i->R);

  if ((y = nsp_matrix_create(NVOID,'c',u->m,u->n)) == NULLMAT) {
    return RET_BUG;
  }

  if (isn==1) {
    for(i=0;i<u->mn;i++) {
      y->C[i].r=u_r->R[i]/u->mn;
      y->C[i].i=u_i->R[i]/u->mn;
    }
  }
  else {
    for(i=0;i<u->mn;i++) {
      y->C[i].r=u_r->R[i];
      y->C[i].i=u_i->R[i];
    }
  }

  MoveObj(stack,1, NSP_OBJECT(y));
  return 1;
}

static OpTab intmodnum_lib_func[]={
  {"genint",intgenint_c},
  {"genrayl",intgenrayl_c},
  {"upsmpl",intsurecht_c},
  {"downsmpl",intsousecht_c},
  {"modpsk",intmodpsk_c},
  {"demodpsk",intdemodpsk_c},
  {"modqam",intmodqam_c},
  {"demodqam",intdemodqam_c},
  {"gengold",intgengold_c},
  {"genpn",intgenpn_c},
  {"spread",intspread_c},
  {"despread",intdespread_c},
  {"nfilter",intnfilter_c},
  {"awgn",intawgn_c},
  {"intsym",intintsym_c},
  {"calc_fft",intcalc_fft_c},
  {(char *) 0, NULL}
};

int intmodnum_lib_Interf(int i, Stack stack, int rhs, int opt, int lhs)
{
  return (*(intmodnum_lib_func[i].fonc))(stack,rhs,opt,lhs);
}

void intmodnum_lib_Interf_Info(int i, char **fname, function (**f))
{
  *fname = intmodnum_lib_func[i].name;
  *f = intmodnum_lib_func[i].fonc;
}



