/* dsm1_c subroutine
 * First order Delta-Sigma Modulator
 *
 * Copyright (C) 2007-2010 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* dsm1_c routine de calcul d'un modulateur sigma-delta du 1er ordre
 *
 * entres :
 * n   : longueur du vecteur d'entre (scalaire)
 * m   : gain du modulateur (scalaire)
 * u   : adresse de dpart du vecteur  convertir 
 * entres/sorties 
 * e   : intgral du signal d'erreur du dernier lment (scalaire)
 * z   : etat de sortie du dernier lment +m;-m (scalaire)
 * y   : adresse de dpart du vecteur de sortie (+1;-1)
 * sorties :
 * q   : adresse de dpart du vecteur de sortie de l'erreur de quantification
 */

void dsm1_c(int *n,double *m,double *u,double *e,double *z,double *y,double *q)
{
 /*dclaration*/
 int i;

 for(i=0;i<(*n);i++)
 {
  /*Calcul de l'intgrale du signal d'erreur*/
  *e+=u[i]-(*z);

  /*Ralisation de la comparaison de signe*/
  if((*e)>0) y[i]=1;
  else y[i]=-1;

  /*mise en mmoire de la sortie*/
  *z = *m*y[i];

  /*Calcul du bruit de quantification*/
  q[i]=(*z)-(*e);
 }
 return;
}

/* dsm2_c routine de calcul d'un modulateur sigma-delta du 2eme ordre
 *
 * entres :
 * n   : longueur du vecteur d'entre (scalaire)
 * m   : gain du modulateur (scalaire)
 * u   : adresse de dpart du vecteur  convertir
 * entres/sorties 
 * e   : intgral du signal d'erreur du dernier lment (vecteur)
 *       e[0]     : modulateur 1
 *       e[1]   : modulateur 2
 * z   : etat de sortie du dernier lment +m;-m (scalaire)
 *       z[0]     : modulateur 1
 *       z[1]     : modulateur 2
 *       z[2]     : etat mmoire du dsm2
 * y   : adresse de dpart du vecteur de sortie (+1;-1)
 * q   : adresse de dpart du vecteur de sortie de l'erreur de quantification
 * w   : adresse de dpart du vecteur de travail
 *
 * dpendances :
 * dsm1_c
 */

void dsm2_c(int *n,double *m,double *u,double *e,double *z,double *y,double *q,double *w)
{
 /*dclaration*/
 int i;

 /*Appel dsm1*/
 dsm1_c(n,m,&u[0],&e[0],&z[0],&y[0],&q[0]);

 /*Appel dsm1*/
 dsm1_c(n,m,&q[0],&e[1],&z[1],&w[0],&q[0]);

 /*ralise opration sur bruit*/
 for(i=0;i<(*n);i++)
 {
  y[i]=(y[i]+z[2]-w[i]);
  /*mise en mmoire de l'etat mmoire du dsm2*/
  z[2]=w[i];
 }
 return;
}

/* dsm3_c routine de calcul d'un modulateur sigma-delta du 3eme ordre
 *
 * entres :
 * n   : longueur du vecteur d'entre (scalaire)
 * m   : gain du modulateur (scalaire)
 * u   : adresse de dpart du vecteur  convertir
 * entres/sorties 
 * e   : intgral du signal d'erreur du dernier lment (vecteur)
 *      e[0]     : modulateur 1
 *      e[1]     : modulateur 2
 *      e[2]     : modulateur 3
 * z   : etat de sortie du dernier lment +m;-m (scalaire)
 *      z[0]     : modulateur 1
 *      z[1]     : modulateur 2
 *      z[2]     : etat mmoire du dsm2
 *      z[3]     : modulateur 3
 *      z[4],z[5]: etats mmoire du dsm3
 * y   : adresse de dpart du vecteur de sortie (+1;-1)
 * q   : adresse de dpart du vecteur de sortie de l'erreur de quantification
 * w   : adresse de dpart du vecteur de travail
 *
 * dpendances :
 * dsm1_c
 * dsm2_c
 */

void dsm3_c(int *n,double *m,double *u,double *e,double *z,double *y,double *q,double *w)
{
 /*dclaration*/
 int i;

 /*Appel dsm2*/
 dsm2_c(n,m,&u[0],&e[0],&z[0],&y[0],&q[0],&w[0]);

 /*Appel dsm1*/
 dsm1_c(n,m,&q[0],&e[2],&z[3],&w[0],&q[0]);

 /*ralise opration sur bruit*/
 for(i=0;i<(*n);i++)
 {
  y[i]=(y[i]+(w[i]-2*z[4]+z[5]));
  /*mise en mmoire des tats mmoires du dsm3*/
  z[5]=z[4];
  z[4]=w[i];
 }
 return;
}
